function IOCoding_MultiLabel_TrainTest_OC_MFA(trX, trY, tsX, tsY, nSample, nRun, indicies, fStr)
% trX: nTr by p matrix of nTr training examples, p features
% trY: nTr by m matrix of nTr training examples, m labels
% tsX: nTs by p matrix of nTs testing examples, p features
% tsY: nTs by m matrix of nTs testing examples, m labels 
% nSample: number of training samples for each run
% nRun: number of random runs
% indicies: nRun * 1 cell array, each cell is 1*nSample permutation vector
% fStr: string for naming the result file

%Basic statistics
nTr = size(trX,1);
nTs = size(tsX,1);
nFea = size(trX,2);
nLabel = size(trY,2);

%corrresponds to \lambda = 4/1/0.25, which balances the Bernoulli and Gaussian potentials
sigmaFcts = [0.5, 1, 2]

%Performance measure
perf_measure = @computeF1;
perf_measure2 = @computeF1_2;

%Number of CCA components used (0 corresponds to no CCA coding)
nComps = [0 : min(nFea,nLabel)]


%result
macroF1_labels = zeros(nRun, length(nComps), nLabel);
macroF1_labels_2 = zeros(nRun, length(nComps), nLabel);
macroF1_labels_3 = zeros(nRun, length(nComps), nLabel);

%macroF1
macroF1 = zeros(nRun, length(nComps));
macroF1_2 = zeros(nRun, length(nComps));
macroF1_3 = zeros(nRun, length(nComps));
%microF1
microF1 = zeros(nRun, length(nComps));
microF1_2 = zeros(nRun, length(nComps));
microF1_3 = zeros(nRun, length(nComps));
%1-exactMatchRate: count as an error if *any* label is predicted incorrectly
ZeroOneLoss = zeros(nRun, length(nComps));
ZeroOneLoss_2 = zeros(nRun, length(nComps));
ZeroOneLoss_3 = zeros(nRun, length(nComps));


for r = 1 : nRun
    
    %learning to predict original labels with logistic regression
    tsY_probs = zeros(size(tsY,1),nLabel);
    for t = 1 : nLabel
        [LR, regulazor, err] = adaptiveTrainLGR_Liblin( trX(indicies{r,1}(1:nSample),:), trY(indicies{r,1}(1:nSample),t), 5, zeros(nFea,1), eye(nFea), perf_measure );
        tsY_probs(:,t) = 1 ./ ( 1 + exp( -1 .* ([ones(size(tsX,1),1), tsX] * LR.w') ) );
    end

    
    %extract the cononical correlation components
    [A,B,cor] = CCA( [ones(nSample,1),trX(indicies{r,1}(1:nSample),:)]',trY(indicies{r,1}(1:nSample),:)');

    %produce the output code
    nc = min(size(B,2),max(nComps));
    CMtx = B(:,1:nc);
    trY_C = trY * CMtx;

    %learning to predict output code with regression
    tsY_C = zeros(size(tsY,1),nc);
    sigma = zeros(1,nc);
    for t = 1 : nc
        [beta lambda optMSE] = adaptiveTrainRLS_Regress_CG( trX(indicies{r,1}(1:nSample),:), trY_C(indicies{r,1}(1:nSample),t), 5 );
        sigma(t) = sqrt(optMSE);        
        %predict the output code for testing examples
        tsY_C(:,t) = [ones(size(tsX,1),1), tsX] * beta;
        %tsY_C(:,t) = [ones(size(tsX,1),1), tsX] * beta + mean(trY_C(indicies{r,1}(1:nSample),t));
    end
    
    %start decoding and evaluation
    for k = 1 : length(nComps)
        nc = min(size(B,2), nComps(k));
        CMtx = B(:,1:nc);

        %decoding
        if nc==0
            tsY_hat = (tsY_probs > 0.5) + 0.0;
            tsY_hat_2 = (tsY_probs > 0.5) + 0.0;
            tsY_hat_3 = (tsY_probs > 0.5) + 0.0;
            %tsY_hat = tsY_probs;
            %tsY_hat_2 = tsY_probs;
            %tsY_hat_3 = tsY_probs;
        else
            tsY_hat = zeros( size(tsX,1), nLabel );
            for i = 1 : size(tsX,1)
                tsY_hat(i,:) = IOC_MFADecoding( tsY_probs(i,:), tsY_C(i,1:nc), sigmaFcts(1)*sigma(1:nc), CMtx );
                %tsY_hat(i,:) = IOC_MFADecoding( tsY_probs(i,:), tsY_C(i,1:nc), sqrt(nc/nLabel)*sigmaFcts(1)*sigma(1:nc), CMtx );
                tsY_hat(i,:) = (tsY_hat(i,:)>=0.5)+0.0;
            end
            %save info_decode tsY_probs tsY_C sigma CMtx tsY_hat
            %tsY_hat = (tsY_hat>=0.5)+0.0;
            %fprintf(1,'total: %d, true pos: %d, original pos: %d, adjusted pos v1: %d\n\n\n', size(tsY_hat,1)*size(tsY_hat,2), nnz(tsY), nnz(tsY_probs>0.5), nnz(tsY_hat>0.5));
            
            tsY_hat_2 = zeros( size(tsX,1), nLabel );
            for i = 1 : size(tsX,1)
                tsY_hat_2(i,:) = IOC_MFADecoding( tsY_probs(i,:), tsY_C(i,1:nc), sigmaFcts(2)*sigma(1:nc), CMtx );
                tsY_hat_2(i,:) = (tsY_hat_2(i,:)>=0.5)+0.0;
            end
            %fprintf(1,'total: %d, true pos: %d, original pos: %d, adjusted pos v2: %d\n\n\n', size(tsY_hat_2,1)*size(tsY_hat_2,2), nnz(tsY), nnz(tsY_probs>0.5), nnz(tsY_hat_2>0.5));
        
            tsY_hat_3 = zeros( size(tsX,1), nLabel );
            for i = 1 : size(tsX,1)
                tsY_hat_3(i,:) = IOC_MFADecoding( tsY_probs(i,:), tsY_C(i,1:nc), sigmaFcts(3)*sigma(1:nc), CMtx );
                tsY_hat_3(i,:) = (tsY_hat_3(i,:)>=0.5)+0.0;
            end
            %fprintf(1,'total: %d, true pos: %d, original pos: %d, adjusted pos v3: %d\n\n\n', size(tsY_hat_3,1)*size(tsY_hat_3,2), nnz(tsY), nnz(tsY_probs>0.5), nnz(tsY_hat_3>0.5));

        end
        
        
        %measure the performance
        for t = 1 : nLabel
            macroF1_labels(r,k,t) = perf_measure2(tsY(:,t), tsY_hat(:,t));           
            macroF1_labels_2(r,k,t) = perf_measure2(tsY(:,t), tsY_hat_2(:,t));
            macroF1_labels_3(r,k,t) = perf_measure2(tsY(:,t), tsY_hat_3(:,t));
        end
        macroF1(r,k) = mean(macroF1_labels(r,k,:));
        macroF1_2(r,k) = mean(macroF1_labels_2(r,k,:));
        macroF1_3(r,k) = mean(macroF1_labels_3(r,k,:));
        
        
        %also measure the microF1
        TP = zeros(3,nLabel); FP = zeros(3,nLabel); TN = zeros(3,nLabel); FN = zeros(3,nLabel);
        for t = 1 : nLabel
            [tempF TP(1,t) FP(1,t) TN(1,t) FN(1,t)] = computeF1_2(tsY(:,t), tsY_hat(:,t)); 
            [tempF TP(2,t) FP(2,t) TN(2,t) FN(2,t)] = computeF1_2(tsY(:,t), tsY_hat_2(:,t)); 
            [tempF TP(3,t) FP(3,t) TN(3,t) FN(3,t)] = computeF1_2(tsY(:,t), tsY_hat_3(:,t)); 
        end       
        microF1(r,k) = computeF1_3( sum(TP(1,:)), sum(FP(1,:)), sum(TN(1,:)), sum(FN(1,:)) );
        microF1_2(r,k) = computeF1_3( sum(TP(2,:)), sum(FP(2,:)), sum(TN(2,:)), sum(FN(2,:)) );
        microF1_3(r,k) = computeF1_3( sum(TP(3,:)), sum(FP(3,:)), sum(TN(3,:)), sum(FN(3,:)) );
        
        %also measure the one-error
        for i = 1 : nTs
            ZeroOneLoss(r,k) = ZeroOneLoss(r,k) + ( nnz(tsY(i,:)~=tsY_hat(i,:))~=0 );
            ZeroOneLoss_2(r,k) = ZeroOneLoss_2(r,k) + ( nnz(tsY(i,:)~=tsY_hat_2(i,:))~=0 );
            ZeroOneLoss_3(r,k) = ZeroOneLoss_3(r,k) + ( nnz(tsY(i,:)~=tsY_hat_3(i,:))~=0 );
        end
        ZeroOneLoss(r,k) = ZeroOneLoss(r,k)./nTs;
        ZeroOneLoss_2(r,k) = ZeroOneLoss_2(r,k)./nTs;
        ZeroOneLoss_3(r,k) = ZeroOneLoss_3(r,k)./nTs;
        
    end
end

SubAccuracy = 1 - ZeroOneLoss;
SubAccuracy_2 = 1 - ZeroOneLoss_2;
SubAccuracy_3 = 1 - ZeroOneLoss_3;

save(sprintf('file_%s_nSample%d_CCACoding_MFADecoding', fStr, nSample), 'microF1', 'microF1_2', 'microF1_3', 'SubAccuracy', 'SubAccuracy_2', 'SubAccuracy_3', 'macroF1', 'macroF1_2', 'macroF1_3', 'nComps', 'sigmaFcts');
