function weights = TrainRLS_Regress_CG( X, Y, lambda, W0, ProjMtx )
% X: N * K+1 data matrix, the augmented X!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
% Y: N * 1 label matrix
% lambda: scalar, regularization coefficient
% W0: (K+1) * 1 prior vector

if nargin < 4
    W0 = zeros(size(X,2), 1);
end

%exclude null columns
oldX = X;
oldW0 = W0;
flags = sum(abs(X));
X = X(:, flags>0);
W0 = W0(flags>0);
if nargin > 4
    ProjMtx = ProjMtx(flags>0, flags>0);
end


%size of data
n = size(X,1);
p = size(X,2);

%Initialize the parameters
weights = unifrnd(-0.0001, 0.0001, p, 1);

%Begin to update weight
products_old = -1 * ones(n, 1);
gradient_old = zeros(p,1);

%calculate some statistics for the first iteration
products = X * weights;


if nargin>4
    gradient = -1 * lambda * ProjMtx * (weights-W0) + X' * (Y-products);
else
    gradient = -1 * lambda * (weights-W0) + X' * (Y-products);
end



iter = 0;
maxIter = 6000;

while max(abs(products - products_old)) > 0.0000001 && iter < maxIter
    iter = iter + 1;
            
    %conjugate gradient
    if iter==1
        cg = gradient;
    else
        %Hestenes-Stifel formula
        beta = (gradient'*(gradient-gradient_old))/(cg'*(gradient-gradient_old));
        cg = gradient - cg*beta;
    end
    
    %project the Newton step into the conjugate gradient
    step = (gradient'*cg)/(lambda*(cg'*cg) + sum((X*cg) .* (X*cg)));
    oldWeights = weights;
    products_old = products;
    gradient_old = gradient;
    
    %update weight and check
    weights = oldWeights + step*cg;
    products = X * weights;
    
    if nargin > 4
        gradient = -1 * lambda * ProjMtx * (weights-W0) + X' * (Y-products);  
        delta_loss = sum((Y-products).^2)+lambda*(norm(ProjMtx*(weights-W0))^2)-sum((Y-products_old).^2)-lambda*(norm(ProjMtx*(oldWeights-W0))^2);
    else
        gradient = -1 * lambda * (weights-W0) + X' * (Y-products);  
        delta_loss = sum((Y-products).^2)+lambda*(norm(weights-W0)^2)-sum((Y-products_old).^2)-lambda*(norm(oldWeights-W0)^2);
    end
        
    %disp(' ');
    %disp(sprintf('Iter: %d',iter));
    %disp(sprintf('Initial change of loss: %d with step %d',  delta_loss, step));
    
    while delta_loss > 0.0000000001
        step = step / 10;
        weights = oldWeights + step*cg;
        products = X * weights;
        if nargin > 4
            gradient = -1 * lambda * ProjMtx * (weights-W0) + X' * (Y-products);  
            delta_loss = sum((Y-products).^2)+lambda*(norm(ProjMtx*(weights-W0))^2)-sum((Y-products_old).^2)-lambda*(norm(ProjMtx*(oldWeights-W0))^2);
        else
            gradient = -1 * lambda * (weights-W0) + X' * (Y-products);  
            delta_loss = sum((Y-products).^2)+lambda*(norm(weights-W0)^2)-sum((Y-products_old).^2)-lambda*(norm(oldWeights-W0)^2);
        end
    end
    
    %disp(sprintf('loss: %d', sum((Y-products).^2)+lambda*norm(weights-W0)));
    
    %dignostic purpose
    %disp(sprintf('Iter: %d',iter));
    %disp(sprintf('Norm of cg: %d', norm(cg)));
    %disp(sprintf('Step: %d', step));
    %disp(sprintf('Norm of g: %d --> %d', norm(gradient_old), norm(gradient)));
    %disp(sprintf('Final change of loss: %d with step %d',  delta_loss, step));
end


oldW0(flags>0) = weights;
weights = oldW0;

%iter
if iter == maxIter
    disp('not enough iter');
end

