function [LR, regulazor, err] = adaptiveTrainLGR_Liblin( X, Y, nFold, prior, ProjMtx, perf_measure )

if nargin < 4
    prior = zeros(size(X,2)+1,1);
elseif size(prior,1) == size(X,2)
    prior = [0; prior];
    %size(prior)
end

if nargin < 5
    ProjMtx = eye(size(X,2)+1);
elseif size(ProjMtx,1) == size(X,2)
    temp = ProjMtx;
    ProjMtx = zeros(size(X,2)+1, size(X,2)+1);
    ProjMtx(1,1) = 1;
    ProjMtx(2:(size(X,2)+1), 2:(size(X,2)+1)) = temp;
    clear temp;
    %size(ProjMtx)
end

if nnz(Y==-1) > 0
    disp('Convert -1 to 0!!');
    Y(Y==-1) = 0;
end

%index postive examples and negative examples
posIndex = find(Y==1);
negIndex = find(Y==0);
nPos = size(posIndex,1);
nNeg = size(negIndex,1);
nFold = min(nFold,min(nPos,nNeg));
posIndex = posIndex(randperm(nPos));
negIndex = negIndex(randperm(nNeg));

%handle the case that nFold = 1 (when only 1 positive or negative example)
if nFold == 1
    regulazor = 1;
    %LR = TrainLogRegress_CG( [ones(size(X,1),1),X], Y, regulazor, prior, ProjMtx );
    LR = train( Y([posIndex;negIndex],:), sparse(X([posIndex;negIndex],:)), sprintf('-s 0 -c %f -e 0.0001 -B 1 -q 1', 1/regulazor) );
    LR.nr_feature = LR.nr_feature+1;
    LR.bias = -1;
    b = LR.w(LR.nr_feature);
    LR.w(2:LR.nr_feature) = LR.w(1:(LR.nr_feature-1));
    LR.w(1) = b;
    err = 0;
    return;
end

%size(X)
%size(Y)

if size(posIndex,1) + size(negIndex,1) ~= size(Y,1)
    disp('Error in LR training, labels are incorrect!!!');
    x = stmesd(1); %stupid way to stop this program
end

for i = 1 : nFold
    posTrain = posIndex([1 : round((i-1)*nPos/nFold), round((i)*nPos/nFold)+1 : nPos]);
    negTrain = negIndex([1 : round((i-1)*nNeg/nFold), round((i)*nNeg/nFold)+1 : nNeg]);
    posTest = posIndex( round((i-1)*nPos/nFold)+1 : round((i)*nPos/nFold) );
    negTest = negIndex( round((i-1)*nNeg/nFold)+1 : round((i)*nNeg/nFold) );
    %disp(sprintf('size of posTrain %g: %g, %g', i, size(posTrain,1), size(posTrain,2)));
    %disp(sprintf('size of negTrain: %g', size(negTrain,1)));
    %disp(sprintf('size of posTest: %g', size(posTest,1)));
    %disp(sprintf('size of negTest: %g', size(negTest,1)));
    trainFold{i} = [posTrain; negTrain];
    testFold{i} = [posTest; negTest];
    %disp(sprintf('size of Train fold %g: (%g, %g)', i, size(trainFold{i},1), size(trainFold{i},2)));
    %disp(sprintf('size of Test fold %g: (%g, %g)', i, size(testFold{i},1), size(testFold{i},2)));
end


%find the best lamda
%lamda = [0.00001, 0.00004, 0.0001, 0.0004, 0.0007, 0.001, 0.004, 0.007, 0.01, 0.04, 0.07, 0.1, 0.4, 0.7, 1.0, 4, 7, 10, 40, 70, 100, 400, 700, 1000, 4000, 7000, 10000, 40000, 100000];
%disp('few lambda');
%lamda = 0.00001*[0.001, 0.004, 0.01, 0.04, 0.1, 0.4, 1.0, 4, 10, 40, 100, 400, 1000, 4000, 10000, 40000, 100000, 400000, 1000000];
%lamda = 10.^[-3 -2.5 -2 -1.5 -1 -0.5  0 0.5 1 1.5 2 2.5 3 3.5 4 4.5 5 5.5 6 6.5 7 8];

%lamda = 10.^[-3 -2.5 -2 -1.5 -1 -0.5  0 0.5 1 1.5 2 2.5 3 3.5 4 4.5 5];

%lamda = 10.^[-3:0.25:6];
lamda = 10 .^ [-6:1:6];


%disp('normal lambda');
%lamda = 0.01* [0.00001, 0.00004, 0.0001, 0.0004, 0.001, 0.004, 0.01, 0.04, 0.1, 0.4, 1.0, 4, 10, 40, 100, 400, 1000, 4000, 10000, 40000, 100000, 500000, 1000000, 5000000];




for i = 1 : size(lamda,2)
    error(i) = 0;
    for j = 1 : nFold
        %size(trainFold{j})
        %size(testFold{j})
        %size(X(trainFold{j},:))
        %size(X(testFold{j},:))
        
        %LogRegress = TrainLogRegress_CG( [ones(size(X(trainFold{j},:),1),1),X(trainFold{j},:)], Y(trainFold{j},:), lamda(i), prior, ProjMtx );
        
        LogRegress = train( Y(trainFold{j},:), sparse(X(trainFold{j},:)), sprintf('-s 0 -c %f -e 0.0001 -B 1 -q 1', 1/lamda(i)) );
        LogRegress.nr_feature = LogRegress.nr_feature+1;
        LogRegress.bias = -1;
        b = LogRegress.w(LogRegress.nr_feature);
        LogRegress.w(2:LogRegress.nr_feature) = LogRegress.w(1:(LogRegress.nr_feature-1));
        LogRegress.w(1) = b;
        
        if nargin<6
            %labels = LogRegress.classify( X(testFold{j},:) );
            [labels tmp2 tmp3] = predict( Y(testFold{j},:), sparse([ones(size(X(testFold{j},:),1),1),X(testFold{j},:)]), LogRegress );
            tempE = nnz(labels~=Y(testFold{j},:))/size(testFold{j},1);
        else
            tempE = 1-perf_measure(X(testFold{j},:), Y(testFold{j},:)*2-1, LogRegress.w');
        end
        
        error(i) = error(i) + tempE;
        %disp(sprintf('This fold has error: %g', tempE));
    end
    error(i) = error(i)/nFold;
    disp(sprintf('Error for lamda %g is averaged as %g', lamda(i), error(i)));
end

%disp('Final error results:');
%disp(error);

%error(1) = (error(1) + error(2)*0.5)/1.5;
%error(size(error,2)) = (error(size(error,2)) + error(size(error,2)-1)*0.5)/1.5;
%for i = 2 : size(error,2)-1
%    error(i) = 0.5 * ( error(i) + 0.5*error(i-1) + 0.5*error(i+1) );
%end
%disp('Smoothed error results:');
%disp(error);

C = min(error);
C2 = min(error(length(error):-1:1));
Is = find(error==C);
I = Is(round(length(Is)/2));
if C~=C2
    C
    C2
    error('C cannot be different to C2');
end
fprintf(1,'LogR-CG, minimum error %g at lamda:', C);
disp(Is);
fprintf(1,'Choose: %d\n', I);

regulazor = lamda(I);
%LR = TrainLogRegress_CG( [ones(size(X,1),1),X], Y, regulazor, prior, ProjMtx );
LR = train( Y([posIndex;negIndex],:), sparse(X([posIndex;negIndex],:)), sprintf('-s 0 -c %f -e 0.0001 -B 1 -q 1', 1/regulazor) );
LR.nr_feature = LR.nr_feature+1;
LR.bias = -1;
b = LR.w(LR.nr_feature);
LR.w(2:LR.nr_feature) = LR.w(1:(LR.nr_feature-1));
LR.w(1) = b;
      
err = error;